'use strict';
const constants = require(".././constants.json")
const config = require('./config.json')
const libSupport = require('./lib')
libSupport.updateConfig()
const node_id = config.id
const {spawn } = require('child_process')
let grunt = spawn('./grunt')
const execute = require('./execute')
const fs = require('fs')
const kafka = require('kafka-node')

const local_repository = __dirname + "/local_repository/"
const host_url = "http://" + constants.master_address + ":" + constants.master_port

let Producer = kafka.Producer,
    client = new kafka.KafkaClient({ 
        kafkaHost: constants.kafka_host,
        autoConnect: true
    }),
    producer = new Producer(client),
    Consumer = kafka.Consumer

libSupport.makeTopic(node_id).then(() => {
    console.log("node topic created")
    let consumer = new Consumer(client,
        [
            { topic: node_id, partition: 0, offset: 0 }
        ],
        [
            { autoCommit: true }
        ])
    consumer.on('message', function (message) {
        console.log(message);
        let topic = message.topic
        message = message.value
        message = JSON.parse(message)
        let runtime = message.runtime
        let functionHash = message.functionHash
        let resource_id = message.resource_id
        let port = message.port
        /**
         * Download necessary files (function file) and Start resource deployment
         */
        if (message.type === "execute") {
            console.log("function_id", resource_id);
            libSupport.download(host_url + "/repository/" + functionHash + ".js", local_repository + functionHash + ".js").then(() => {
                startWorker(local_repository, functionHash, resource_id, producer, runtime, port)
            })

        }

    })
})
    
/**
 * Start a worker executor of the runtime type
 * @param {String} local_repository 
 * @param {String} functionHash 
 * @param {String} resource_id 
 * @param {String} producer 
 * @param {String} runtime 
 * @param {Number} port 
 */
function startWorker(local_repository, functionHash, resource_id, producer, runtime, port) {
    console.log("Using port", port, "for functionHash", functionHash);
    
    usedPort.set(port, functionHash)
    fs.writeFile('./local_repository/config.json', JSON.stringify({port, functionHash, resource_id, runtime}), () => {
        if (runtime === "isolate")
            execute.runIsolate(local_repository, functionHash, port, resource_id)
        else if (runtime === "process")
            execute.runProcess(local_repository, functionHash, port, resource_id)
        else if (runtime === "container")
            execute.runContainer(functionHash, port, resource_id)
        else {
            producer.send(
                [{
                    topic: "response",
                    messages: JSON.stringify({ status: "unknown runtime" })
                }], () => { })

            return
        }
    });
    
}

function heartbeat() {
    let payload = [{
        topic: "heartbeat",
        messages: JSON.stringify({"address": node_id, "timestamp": Date.now()})
    }]
    producer.send(payload, function() {})
}


grunt.stdout.on('data', data => {
    console.log(data.toString());
    
})

grunt.stderr.on('data', data => {
    console.log(data.toString());
    
})


grunt.on('close', (code) => {
    console.log("Grunt exited with exit code", code);
    
})


setInterval(heartbeat, 1000);
/**
 * Channel LOG_COMMON
  Source: Executor
  {
    "node_id"
    "resource_id"
    "function_id"
    "status": true/false
    "reason": "deployed / exd"
 }
 Source: Executor
 {
    "node_id"
    "resource_id"
    "function_id"
    "usage": {
        "cpu"
        "memory"
        "network"
    }
}
Source: ReverseProxy
{
    "node_id"
    "resource_id"
    "function_id"
    "average_fn_time"
}
Source: Dispatch Manager
{
    "node_id"
    "resource_id"
    "function_id"
    "coldstart_time" 
}
 */
